/*
 * LinRemote : a program to emulate Mindstorms Remote under linux
 * Copyright (C) 2003  nanobapt
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */


/* 
 * IO fonction
 */
 
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <stdlib.h>
#include "IO.h"

#define TRUE 1
#define FALSE 0

int iTTY(char *name)
{
	
	/* 
	 * Init TTY
	 */
	
	struct termios ios;
	int fd;
	
	if(name == NULL)
	{
		name = malloc(strlen(DEFAULT_TTY));
		strcpy(name, (char *)DEFAULT_TTY);
	}

    if ((fd = open(name, O_RDWR|O_NONBLOCK)) < 0)  // open it
    {
		perror("open");
		exit(1);
    }

	// configure it

	memset(&ios, 0, sizeof(ios));
	ios.c_cflag = CREAD | CLOCAL | CS8 | PARENB | PARODD;
    cfsetispeed(&ios, B2400);
    cfsetospeed(&ios, B2400);

    if (tcsetattr(fd, TCSANOW, &ios) == -1) 
    {
		perror("tcsetattr");
		exit(1);
	}
	return fd;
}

unsigned char *send(unsigned char code[10],int lenght, int fd, int ans)
{

	/*
	 * Send RCX message function
	 */        	
	        
	int d, count;
	unsigned char *buffer;
	unsigned char *rcv;
	fd_set fds;
    struct timeval tv;

	buffer=malloc((2*lenght)+5);

    buffer[0] = 0x55;  // headers
    buffer[1] = 0xff;
    buffer[2] = 0x00;

	for(d=0; d<lenght; d++)
	{
		buffer[(2*d)+3] = code[d];
		buffer[(2*d)+4] = 0xFF - code[d];
	}
        
    buffer[3+(2*lenght)] = 0x00;
        
	for(d=0; d<lenght; d++)
		buffer[3+(2*lenght)] = code[d]+buffer[3+(2*lenght)];
			
	buffer[4+(2*lenght)]=0xFF - buffer[3+(2*lenght)];
	
#ifdef DEBUG
	for(d=0; d<(2*lenght)+5; d++)
		printf("%X - ", buffer[d]);        	
        	
	printf("\n");
#endif

	write(fd, buffer, (2*lenght)+5);	
	
	if(ans==TRUE)
	{
		usleep(100000);
	
		FD_ZERO(&fds);
		FD_SET(fd, &fds);

		tv.tv_sec = 1;
		tv.tv_usec = 0;

		if (select(fd+1, &fds, NULL, NULL, &tv) < 0) 
		{
			perror("select");
			exit(1);
		}

		if ((count = read(fd, &buffer[0], (2*lenght)+5)) < 0) 
		{
			perror("read");
			exit(1);
		}
	
#ifdef DEBUG
		printf("Echo :");
		for(d=0; d<(2*lenght)+5; d++)
			printf("%X - ", buffer[d]);
        	
		printf("\n");
#endif

		//Receive answer of RCX
		
		usleep(100000);
		
		free(buffer);
		
		buffer = malloc(50);

		FD_ZERO(&fds);
		FD_SET(fd, &fds);

		tv.tv_sec = 1;
		tv.tv_usec = 0;

		if (select(fd+1, &fds, NULL, NULL, &tv) < 0) 
		{
			perror("select");
			exit(1);
		}

		if ((count = read(fd, buffer, 50)) < 0) 
		{
			perror("read");
			exit(1);
		}

		if(buffer[0]==0x55 && buffer[1]==0xFF && buffer[2]==0x00)
		{
			int i=0;
			unsigned char tmp=0x00;
#ifdef DEBUG			
			printf("Recv :");
#endif
			for(d=3; d<50; d++)
			{
				if(d%2)
				{
					tmp=tmp+buffer[d];
					if(tmp==buffer[d+2])
					{
						d=55; //end of RCX reply
						i++;
					}
				}
				i++;
#ifdef DEBUG
				printf("%X - ", buffer[d]);
#endif
			}
#ifdef DEBUG		
			printf("\n");
#endif
			//Analyse the answer
			if(d==56)
			{
				//good answer of RCX
				rcv=malloc((i/2)+1);
				rcv[0]=i/2; //len of reponse
				
#ifdef DEBUG
				printf("Return : ");
#endif
				for(d=0; d<(i/2); d++)
				{
#ifdef DEBUG
					printf("%X -", buffer[d*2+3]);
#endif
					rcv[d+1]=buffer[(d*2+3)];
					if(buffer[d*2+4] != 0xFF-buffer[(d*2+3)])
					{
						printf("Bad IR link\n)");
						return NULL;
					}
				}
#ifdef DEBUG
				printf("\n");
#endif
			return rcv;
			}
			else printf("Bad IR link\n");
		}
		else if(buffer[0]==0x00)
			{
			printf("No reponse from RCX\n");
			return NULL;
			}
		else 
		{
			printf("Bad IR link\n");
			return NULL;
		}
	}

	free(buffer);

#ifdef DEBUG
	printf("end tr/rc\n");
#endif
	return NULL;
}
