/*
 * LinRemote : a program to emulate Mindstorms Remote under linux
 * Copyright (C) 2003  nanobapt
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <stdlib.h>

/*
 * Const declaration
 */

#define DEFAULT_TTY "/dev/ttyS0" //change it if want to use an other
#define MESSAGE_1 0
#define MESSAGE_2 1
#define MESSAGE_3 2
#define FWD_A 3
#define FWD_B 4
#define FWD_C 5
#define REV_A 6
#define REV_B 7
#define REV_C 8
#define STOP 9
#define P1 10
#define P2 11
#define P3 12
#define P4 13
#define P5 14
#define BEEP 15

int fd;
unsigned char code[10];
unsigned char alt=0x00;

void iTTY(void)
{
	
	/* 
	 * Init TTY
	 */
	
	struct termios ios;

    if ((fd = open(DEFAULT_TTY, O_RDWR)) < 0)  // open it
    {
		perror("open");
		exit(1);
    }

	// configure it

	memset(&ios, 0, sizeof(ios));
	ios.c_cflag = CREAD | CLOCAL | CS8 | PARENB | PARODD;
    cfsetispeed(&ios, B2400);
    cfsetospeed(&ios, B2400);

    if (tcsetattr(fd, TCSANOW, &ios) == -1) 
    {
		perror("tcsetattr");
		exit(1);
	}
}

void send(int lenght)
{

	/*
	 * Send RCX message function
	 */        	
	        
	int d;

	unsigned char buffer[20];

    buffer[0] = 0x55;  // headers
    buffer[1] = 0xff;
    buffer[2] = 0x00;

	for(d=0; d<lenght; d++)
	{
		buffer[(2*d)+3] = code[d];
		buffer[(2*d)+4] = 0xFF - code[d];
	}
        
    buffer[3+(2*lenght)] = 0x00;
        
	for(d=0; d<lenght; d++)
		buffer[3+(2*lenght)] = code[d]+buffer[3+(2*lenght)];
			
	buffer[4+(2*lenght)]=0xFF - buffer[3+(2*lenght)];
        
	for(d=0; d<(2*lenght)+5; d++)
		printf("%X - ", buffer[d]);
        	
        	
	printf("\n");

	write(fd, buffer, (2*lenght)+5);
}

static void releas( GtkWidget *widget, gpointer   data )
{
	if(alt == 0x00)
		alt=0x08;
	else
		alt=0x00;
	
	if(data == (gpointer)FWD_A ||data == (gpointer)REV_A)
	{
		code[0]=0x21+alt;
		code[1]=0x41;
		send(2);
	}
	else if(data == (gpointer)FWD_B ||data == (gpointer)REV_B)
	{
		code[0]=0x21+alt;
		code[1]=0x42;
		send(2);
	}
	else if(data == (gpointer)FWD_C ||data == (gpointer)REV_C)
	{
		code[0]=0x21+alt;
		code[1]=0x44;
		send(2);
	}
}

static void callback( GtkWidget *widget, gpointer   data )
{
	/*
	 * Callback function
	 * 	Just send the correct message to RCX
	 */
	
	if(alt == 0x00)
		alt=0x08;
	else
		alt=0x00;
		
	if(data == (gpointer)MESSAGE_1)
	{
		code[0]=0xF7+alt;
		code[1]=0x00;
		send(2);
	}
	else if(data == (gpointer)MESSAGE_2)
	{
		code[0]=0xF7+alt;
		code[1]=0x01;
		send(2);
	}
	else if(data == (gpointer)MESSAGE_3)
	{
		code[0]=0xF7+alt;
		code[1]=0x02;
		send(2);
	}
	else if(data == (gpointer)FWD_A)
	{
		code[0]=0xE1+alt;
		code[1]=0x81;
		send(2);
		usleep(100000);
		code[0]=0x21+alt;
		code[1]=0x81;
		send(2);
	}
	else if(data == (gpointer)FWD_B)
	{
		code[0]=0xE1+alt;
		code[1]=0x82;
		send(2);
		usleep(100000);
		code[0]=0x21+alt;
		code[1]=0x82;
		send(2);
	}
	else if(data == (gpointer)FWD_C)
	{
		code[0]=0xE1+alt;
		code[1]=0x84;
		send(2);
		usleep(100000);
		code[0]=0x21+alt;
		code[1]=0x84;
		send(2);
	}
	else if(data == (gpointer)REV_A)
	{
		code[0]=0xE1+alt;
		code[1]=0x81;
		send(2);
		usleep(100000);
		
		if(alt == 0x00)
			alt=0x08;
		else
			alt=0x00;

		code[0]=0xE1+alt;
		code[1]=0x41;
		send(2);
		usleep(100000);
			
		code[0]=0x21+alt;
		code[1]=0x81;
		send(2);		
	}
	else if(data == (gpointer)REV_B)
	{
		code[0]=0xE1+alt;
		code[1]=0x82;
		send(2);
		usleep(100000);
		
		if(alt == 0x00)
			alt=0x08;
		else
			alt=0x00;

		code[0]=0xE1+alt;
		code[1]=0x42;
		send(2);
		usleep(100000);
		
		code[0]=0x21+alt;
		code[1]=0x82;
		send(2);
	}
	else if(data == (gpointer)REV_C)
	{
		code[0]=0xE1+alt;
		code[1]=0x84;
		send(2);
		usleep(100000);

		if(alt == 0x00)
			alt=0x08;
		else
			alt=0x00;

		code[0]=0xE1+alt;
		code[1]=0x44;
		send(2);
		usleep(100000);

		code[0]=0x21+alt;
		code[1]=0x84;
		send(2);
	}
	else if(data == (gpointer)STOP)
	{
		if(alt == 0x00)
			alt=0x08;
		else
			alt=0x00;
			
		code[0]=0x21+alt;
		code[1]=0x41;
		send(2);
		usleep(100000);
		
		if(alt == 0x00)
			alt=0x08;
		else
			alt=0x00;
			
		code[0]=0x21+alt;
		code[1]=0x42;
		send(2);
		usleep(100000);
		
		if(alt == 0x00)
			alt=0x08;
		else
			alt=0x00;
		
		code[0]=0x21+alt;
		code[1]=0x44;
		send(2);
	}
	else if(data == (gpointer)BEEP)
	{
		code[0]=0x51+alt;
		code[1]=0x00;
		send(2);
	}
	else if(data == (gpointer)P1)
	{
		code[0]=0xD2;
		code[1]=0x00;
		code[2]=0x40;
		send(3);
		usleep(100000);
		code[0]=0xD2;
		code[1]=0x00;
		code[2]=0x00;
		send(3);
	}
	else if(data == (gpointer)P2)
	{
		code[0]=0x91+alt;
		code[1]=0x01;
		send(2);
	}
	else if(data == (gpointer)P3)
	{
		code[0]=0x91+alt;
		code[1]=0x02;
		send(2);
	}
	else if(data == (gpointer)P4)
	{
		code[0]=0x91+alt;
		code[1]=0x03;
		send(2);
	}
	else if(data == (gpointer)P5)
	{
		code[0]=0x91+alt;
		code[1]=0x04;
		send(2);
	}	
	
	if(data == (gpointer)P1 || data == (gpointer)P2 || data == (gpointer)P3 || data == (gpointer)P4 || data == (gpointer)P5)
	{
		usleep(100000);
		code[0]=0x51+alt;
		code[1]=0x00;
		send(2);
	}
}


void cWin() 
{
	/*
	 * Here is program the window 
	 */
	
	//some declaration
	
    GtkWidget *window;
    GtkWidget *button;
    GtkWidget *table;
    GtkWidget *table2;
    GtkWidget *label;
 
	// First create window
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), "Linux MindRemote"); //window title
    gtk_window_set_default_size(GTK_WINDOW(window), 150, 10); // size of window
    g_signal_connect (G_OBJECT (window), "delete_event", G_CALLBACK(gtk_main_quit), NULL); // if want to quit ...
    gtk_container_set_border_width (GTK_CONTAINER (window), 5); // a little border
    
    // Create a big table with all component
    
    table2 = gtk_table_new (10, 1, TRUE);
    gtk_container_add (GTK_CONTAINER (window), table2);
    
    label=gtk_label_new("Message"); // message label
	gtk_table_attach_defaults(GTK_TABLE(table2), label, 0, 1, 0, 1); //put it in table
	gtk_widget_show(label); //display it

	/*
	 * Create the 1st row bouton (message)
	 */
	
    table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 1, 2);

    button = gtk_button_new_with_label ("1");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) MESSAGE_1);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
	gtk_widget_show (button); // montrer le bouton
	
	button = gtk_button_new_with_label ("2");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) MESSAGE_2);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
	gtk_widget_show (button); // montrer le bouton
	
	button = gtk_button_new_with_label ("3");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) MESSAGE_3);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
	gtk_widget_show (button); // montrer le bouton
	
	gtk_table_set_col_spacings(GTK_TABLE(table), 5); //espacement entre les cases
	
    gtk_widget_show (table);
    
    //show 1st row
    
    /*
     * Motors buttons
     */
    
    table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 3, 4);
    
    label=gtk_label_new("A");
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_widget_show(label);
	
	label=gtk_label_new("B");
	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
	gtk_widget_show(label);
	
	label=gtk_label_new("C");
	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
	gtk_widget_show(label);
	
	gtk_widget_show (table);
	
	table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 4, 5);
	
	button = gtk_button_new_with_label ("/\\");
    g_signal_connect (G_OBJECT (button), "pressed", G_CALLBACK (callback), (gpointer) FWD_A);
    g_signal_connect (G_OBJECT (button), "released", G_CALLBACK (releas), (gpointer) FWD_A);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("/\\");
    g_signal_connect (G_OBJECT (button), "pressed", G_CALLBACK (callback), (gpointer) FWD_B);
    g_signal_connect (G_OBJECT (button), "released", G_CALLBACK (releas), (gpointer) FWD_B);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("/\\");
    g_signal_connect (G_OBJECT (button), "pressed", G_CALLBACK (callback), (gpointer) FWD_C);
    g_signal_connect (G_OBJECT (button), "released", G_CALLBACK (releas), (gpointer) FWD_C);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
	gtk_widget_show (button); 
	
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	
	// reverse bouton
	
	gtk_widget_show(table);
	
	table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 5, 6);
	
	button = gtk_button_new_with_label ("\\/");
    g_signal_connect (G_OBJECT (button), "pressed", G_CALLBACK (callback), (gpointer) REV_A);
    g_signal_connect (G_OBJECT (button), "released", G_CALLBACK (releas), (gpointer) REV_A);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("\\/");
    g_signal_connect (G_OBJECT (button), "pressed", G_CALLBACK (callback), (gpointer) REV_B);
    g_signal_connect (G_OBJECT (button), "released", G_CALLBACK (releas), (gpointer) REV_B);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("\\/");
    g_signal_connect (G_OBJECT (button), "pressed", G_CALLBACK (callback), (gpointer) REV_C);
    g_signal_connect (G_OBJECT (button), "released", G_CALLBACK (releas), (gpointer) REV_C);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
	gtk_widget_show (button);
	
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    
    gtk_widget_show (table);
    
    /*
     * Program button
     */
    
    table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 7, 8);
	
	button = gtk_button_new_with_label ("P1");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) P1);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
	gtk_widget_show (button); 
	
	label = gtk_label_new("Program");
	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 0, 1);
	gtk_widget_show (label); 
	
	button = gtk_button_new_with_label ("P2");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) P2);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
	gtk_widget_show (button); 
	
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	
	gtk_widget_show(table);
	
	table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 8, 9);
	
	button = gtk_button_new_with_label ("P3");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) P3);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("P4");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) P4);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("P5");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) P5);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
	gtk_widget_show (button);
	
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    
    gtk_widget_show (table);
    
	table = gtk_table_new (1, 3, TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table2), table, 0, 1, 9, 10);
	
	/*
	 * Other button
	 */
	
	button = gtk_button_new_with_label ("Stop");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) STOP);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 0, 1);
	gtk_widget_show (button); 
	
	button = gtk_button_new_with_label ("Beep");
    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (callback), (gpointer) BEEP);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
	gtk_widget_show (button);
	
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    
    gtk_widget_show (table);
    
    gtk_widget_show (table2);
    gtk_widget_show (window); //display window
}

int main(int argc, char *argv[] )
{
    gtk_init (&argc, &argv); //gtk init
    
    iTTY(); //init TTY "/dev/ttyS0 is the default

	cWin(); //create and display window

    gtk_main (); //gtk loop
    
    close(fd); //close TTY

    return EXIT_SUCCESS;
}